/** @file fifo_media.cpp
 * CPPTESK mediator of FIFO's test system.
 *
 * @author   (chupilko@ispras.ru)
 */

#include <fifo_media.h>

#include <sync.h>

#include <interface.h>
#include <vpi_media.h>

namespace cpptesk {
namespace examples {
namespace fifo {

FIFOMediator::FIFOMediator()
{
    SET_DEBUG_LEVEL(cpptesk::hw::DEBUG_USER, true);
    SET_DEBUG_STYLE(cpptesk::hw::DEBUG_USER, cpptesk::hw::BLACK, cpptesk::hw::CYAN, cpptesk::hw::CYAN, cpptesk::hw::BLACK);

    //   ,   .
    CPPTESK_SET_INPUT_ADAPTER(iface1, FIFOMediator::serialize_iface1);
    CPPTESK_SET_INPUT_ADAPTER(iface2, FIFOMediator::serialize_iface2);
    CPPTESK_SET_OUTPUT_ADAPTER(iface3, FIFOMediator::deserialize_iface3);

    //     .
    SET_REACTION_TIMEOUT(FIFOMediator::REACTION_TIMEOUT);

    //  ,      .
    clear_inputs(&inputs);
    clear_outputs(&outputs);

    //     iface3.
    CPPTESK_SET_ARBITER(iface3, matcher_iface3);

    //        .
    CALL_OUTPUT_LISTENER(FIFOMediator::listen_iface3, iface3);
}

FIFOMediator::~FIFOMediator() {}

bool FIFOMediator::isIface1Ready() const
{
    return FIFO::isIface1Ready();
}
bool FIFOMediator::isIface2Ready() const
{
    return FIFO::isIface2Ready();
}

CPPTESK_DEFINE_PROCESS(FIFOMediator::serialize_iface1)
{
    //     .
    InputData msg = CPPTESK_CAST_MESSAGE(InputData);

    //   .
    START_PROCESS();

    CPPTESK_DEBUG_PRINT(cpptesk::hw::DEBUG_USER, "FIFOExtMediator::serialize_iface1");

    //   iface1.
    CAPTURE_IFACE();

    //       .
    inputs.CE_WR = 1;
    inputs.DI = msg.get_data();

    //   iface1.
    RELEASE_IFACE();

    //   .
    CYCLE();

    //   .
    STOP_PROCESS();
}

CPPTESK_DEFINE_PROCESS(FIFOMediator::serialize_iface2)
{
    //     .
    InputData msg = CPPTESK_CAST_MESSAGE(InputData);

    //   .
    START_PROCESS();

    CPPTESK_DEBUG_PRINT(cpptesk::hw::DEBUG_USER, "FIFOExtMediator::serialize_iface2");

    //   iface2.
    CAPTURE_IFACE();

    //       .
    inputs.CE_RD = 1;

    //   iface1.
    RELEASE_IFACE();

    //   .
    CYCLE();

    //   .
    STOP();
}

CPPTESK_DEFINE_PROCESS(FIFOMediator::deserialize_iface3)
{
    //     .
    OutputData &msg = CPPTESK_CAST_MESSAGE(OutputData);

    //   .
    START_PROCESS();

    CPPTESK_DEBUG_PRINT(cpptesk::hw::DEBUG_USER, "FIFOExtMediator::deserialize_iface3");

    //     ,    .
    WAIT_REACTION(outputs.DO_VAL);

    //       .
    msg.set_data(outputs.DO);

    //     ,   .
    NEXT_REACTION();

    //   .
    STOP_PROCESS();
}

}}}

