/* Copyright 2010-2012 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Verdict generation subsystem main file
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/event.hpp>
#include <hw/interface.hpp>

#include <hw/diagnostics/error_model.hpp>
#include <hw/diagnostics/reaction_set.hpp>
#include <hw/diagnostics/reaction_pair.hpp>
#include <hw/diagnostics/reaction_group.hpp>

namespace cpptesk {
namespace hw {
namespace diagnostics {

class Diagnostics
{
public:
    Diagnostics(const event_list_t &reactions);
    virtual ~Diagnostics() {}

    /// Start the verdict generator
    void run();
    /// Print result of verdict generation
    void printVector(bool debug = false);
    /// Print result of diagnostic generation
    void printDiag();
    
protected:
    /// List of rules
    ErrorModel _errorModel;
    /// List of reaction pairs
    ReactionSet _reactionSet;
};

}}}
