/*Copyright 2010-2012 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: 
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include "reaction_unit.hpp"

#include "rule/rule.hpp"

#include "rule/correct.hpp"
#include "rule/associative.hpp"
#include "rule/union.hpp"
#include "rule/spec_intersection.hpp"
#include "rule/impl_intersection.hpp"
#include "rule/incomplete_associative.hpp"

#include "rule/fuzzy_associative.hpp"
#include "rule/fuzzy_spec_intersection.hpp"
#include "rule/fuzzy_impl_intersection.hpp"
#include "rule/fuzzy_union.hpp"

namespace cpptesk {
namespace hw {
namespace diagnostics {

struct FuzzyMatchResult {
    int weight;
    int rule_number;
};

/*struct MessageError {
    error_type_t error_type;
    int ruleNumber;
};*/

struct ReactionUnitPair {
    ReactionUnit *r1;
    ReactionUnit *r2;
    int r1_id; // identifier of the first reaction
    int r2_id; // identifier of the second reaction
};

class ErrorModel: 
      public Rule
{
    public: 
        ErrorModel() {};
        virtual ~ErrorModel() {};

    unsigned int priority;  
    
    /*ErrorModel::ErrorModel()
    {  //Empty
    }

    ErrorModel::~ErrorModel()
    {  //Empty
    }*/

    void addRule(Rule *rule)
    // Add rule to the ruleSet 
    {
        //rule->setRuleId(_rule_set.size() + _fuzzyRuleSet.size());
        _rule_set.push_back(rule);
    }

    void addFuzzyRule(Rule *fuzzy_rule)
    // Add rule to the fuzzyRuleSet 
    {
        //fuzzyRule->setRuleId(_rule_set.size() + _fuzzy_rule_set.size());
        _fuzzy_rule_set.push_back(fuzzy_rule);
    }

    void initialization()
    {
        // Create normal rule set        
        CorrectReactionEliminationRule *_rule_1 = new CorrectReactionEliminationRule();
        addRule(_rule_1);
        AssociativeEliminationRule *_rule_2 = new AssociativeEliminationRule();
        addRule(_rule_2);
        UnionEliminationRule *_rule_3 = new UnionEliminationRule();
        addRule(_rule_3);
        SpecIntersectionEliminationRule *_rule_4 = new SpecIntersectionEliminationRule();
        addRule(_rule_4);
        ImplIntersectionEliminationRule *_rule_5 = new ImplIntersectionEliminationRule();
        addRule(_rule_5);
        IncompleteAssociativeEliminationRule *_rule_6 = new IncompleteAssociativeEliminationRule();
        addRule(_rule_6);
        
        // Create fuzzy rule set        
        FuzzyAssociativeEliminationRule  *_fuzzy_rule_1 = new FuzzyAssociativeEliminationRule();
        addFuzzyRule(_fuzzy_rule_1);
        FuzzyUnionEliminationRule  *_fuzzy_rule_2 = new FuzzyUnionEliminationRule();
        addFuzzyRule(_fuzzy_rule_2);
        FuzzyImplIntersectionEliminationRule  *_fuzzy_rule_3 = new FuzzyImplIntersectionEliminationRule();
        addFuzzyRule(_fuzzy_rule_3);
        FuzzySpecIntersectionEliminationRule  *_fuzzy_rule_4 = new FuzzySpecIntersectionEliminationRule();
        addFuzzyRule(_fuzzy_rule_4);
    }

    void removeRules()
    {
        _rule_set.clear();
        _fuzzy_rule_set.clear();
        // TODO: use delete for newly allocated objects
    }

    ReactionSet transform(::cpptesk::hw::diagnostics::ReactionUnit &_reaction_unit_1,
                          ::cpptesk::hw::diagnostics::ReactionUnit &_reaction_unit_2)
    {
        int _match_rule = match(_reaction_unit_1.getReaction(), _reaction_unit_2.getReaction());

        ReactionSet _transform_out;

        if(_match_rule != EMPTY_RULE)
        {
            Rule* _rule;

            _rule = _rule_set[_match_rule]; 
            _transform_out = _rule->transform(_reaction_unit_1, _reaction_unit_2);
            _rule->setCollapseRule(_reaction_unit_1, _reaction_unit_2, _match_rule, 
                                   cpptesk::hw::diagnostics::ReactionUnit::COLLAPSE_TYPE_NORMAL);
        }

        return _transform_out; 
    }

    ReactionSet fuzzyTransform(::cpptesk::hw::diagnostics::ReactionUnit &_reaction_unit_1, 
                               ::cpptesk::hw::diagnostics::ReactionUnit &_reaction_unit_2,
                               int _fuzzy_match_rule)     // Collapse the reactions
    {
        //int _fuzzy_match_rule = this->fuzzyMatch(_reaction_unit_1._reaction, _reaction_unit_2._reaction);
        
        ReactionSet _transform_out;

        //if(_fuzzy_match_rule != NOT_FOUND_COLLAPSE_RULE)
        //{
            Rule* _fuzzyRule;

            _fuzzyRule = _fuzzy_rule_set[_fuzzy_match_rule]; 

            _transform_out = _fuzzyRule->transform(_reaction_unit_1, _reaction_unit_2);
            _fuzzyRule->setCollapseRule(_reaction_unit_1, _reaction_unit_2, _fuzzy_match_rule, 
                                   cpptesk::hw::diagnostics::ReactionUnit::COLLAPSE_TYPE_FUZZY);
            _reaction_unit_1.setCollapsed();
        //}
        
        return _transform_out; 
    }

    std::string getRuleDescription(int _rule_index)
    {
        Rule* _rule;
        _rule = _rule_set[_rule_index]; 

        std::string _description = _rule->getRuleDescription();
        return _description;
    }

    std::string getFuzzyRuleDescription(int rule_index)
    {
        Rule* rule;
        rule = _fuzzy_rule_set[rule_index]; 

        std::string _description = rule->getRuleDescription();
        return _description;
    }


    int match(::cpptesk::hw::Reaction &_reaction_1, ::cpptesk::hw::Reaction &_reaction_2)
    // Compare the reactions
    {
        int _match_value = FUZZY_MATCH;
        int i_rule = EMPTY_RULE;
        for(unsigned int i=0; i<_rule_set.size(); i++)
        {
             Rule* _rule = _rule_set[i];

             if(_reaction_1.getSpecMessage() == NULL && _reaction_1.getImplMessage() == NULL) continue;
             if(_reaction_2.getSpecMessage() == NULL && _reaction_2.getImplMessage() == NULL) continue;

             int _match_value_new = _rule->match(_reaction_1, _reaction_2);
             if(_match_value_new < _match_value)
             {
                 _match_value = _match_value_new;
                 i_rule = i; //found collapse rule
             }

        }
        return i_rule;
    }

    FuzzyMatchResult fuzzyMatch(::cpptesk::hw::Reaction &_reaction_1,
                                ::cpptesk::hw::Reaction &_reaction_2)
    {
        FuzzyMatchResult result = {FUZZY_MATCH, EMPTY_RULE};
        int weight;

        for(unsigned int i = 0; i < _fuzzy_rule_set.size(); i++)
        {
            if(_reaction_1.getSpecMessage() == NULL && _reaction_1.getImplMessage() == NULL) continue;
            if(_reaction_2.getSpecMessage() == NULL && _reaction_2.getImplMessage() == NULL) continue;

            weight = (_fuzzy_rule_set[i])->match(_reaction_1, _reaction_2);

            if(weight < result.weight)
            {
                result.weight = weight;
                result.rule_number = i; // found collapse rule
            }
        }

        return result;
    }

    void weight()
    // Get the weight of operation
    {
    }

    void distance()
    //
    {
    }
    
protected:
    // For normal rules
    std::vector<Rule*> _rule_set; 
    // For fuzzy rules
    std::vector<Rule*> _fuzzy_rule_set; 
};

}}}
