/*Copyright 2010-2012 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: 
///////////////////////////////////////////////////////////////////////////////

#include <hw/diagnostics/reaction_pair.hpp>

namespace cpptesk {
namespace hw {
namespace diagnostics {

void ReactionPair::findSimilarity()
{
    setDataError(false);
    setIfaceError(false);
    setTimeError(false);
    _error = 0;
    
    // Failure Data
    Reaction collapsed_reaction = _collapsed_reaction_unit.getReaction();
    
    const ::cpptesk::hw::Message *_message_spec = collapsed_reaction.getSpecMessage();
    const ::cpptesk::hw::Message *_message_impl = collapsed_reaction.getImplMessage();
    
    bool out = true;
    for(::cpptesk::hw::Message::const_iterator i = _message_spec->begin(), j = _message_impl->begin();
            i != _message_spec->end(); i++, j++)
    {
        const Field<> &field_spec = (*_message_spec).**i;
        const Field<> &field_impl = (*_message_impl).**j;

        if((!field_spec.equals(field_impl))&&out)
        {
            out = false;
            setDataError();
            _error = _error + DATA_ERROR;
        }
    }

    if(isCollapsed())
    {
        Reaction impl_reaction = _impl_reaction_unit.getReaction();
        Reaction spec_reaction = _spec_reaction_unit.getReaction();
        
        // Failure iface
        const ::cpptesk::hw::Interface *iface_impl = impl_reaction.where();
        const ::cpptesk::hw::Interface *iface_spec = spec_reaction.where();
        if(iface_impl != iface_spec) 
        { 
            setIfaceError(); 
            _error = _error + IFACE_ERROR;
        }

        // Failure Time
        //int spec_time_starred =0, impl_time_started = 0, spec_time_stopped =0, impl_time_stopped = 0;
        if((impl_reaction.whenStarted() > spec_reaction.whenStopped())||((spec_reaction.whenStarted() > impl_reaction.whenStopped())))
        {
            setTimeError();
            _error = _error + TIME_ERROR;
        }
    }
    
    setGroup(_error);
}

}}}
