/*Copyright 2010-2012 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: 
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <vector>

#include <hw/message.hpp>
#include <hw/diagnostics/reaction_unit.hpp>

namespace cpptesk {
namespace hw {
namespace diagnostics {

class ReactionPair
{
public:
    const static int DATA_TIME_IFACE_ERROR = 7; // 7 
    const static int DATA_TIME_ERROR       = 6; // 6
    const static int DATA_IFACE_ERROR      = 5; // 5
    const static int TIME_IFACE_ERROR      = 3; // 3
    const static int DATA_ERROR            = 4; // 4
    const static int TIME_ERROR            = 2; // 2
    const static int IFACE_ERROR           = 1; // 1
    const static int NONE_ERROR            = 0; // 0

    ReactionPair()
    {
        setGroup(-1);
        setDataError(false);
        setTimeError(false);
        setIfaceError(false);
        setCollapsed(false);
    }
    
    ReactionPair(ReactionUnit &_reaction_impl, ReactionUnit &_reaction_spec, ReactionUnit &_reaction_collapsed):
    _impl_reaction_unit(_reaction_impl.getReaction()), _spec_reaction_unit(_reaction_spec.getReaction()), _collapsed_reaction_unit(_reaction_collapsed.getReaction())
    {
        setGroup(-1);
        setDataError(false);
        setTimeError(false);
        setIfaceError(false);
        setCollapsed(false);
    }

    void setCollapsed(bool collapsed) {_collapsed = collapsed;}
    void setCollapsed() {setCollapsed(true);}
    bool isCollapsed() {return _collapsed;}

    void setDataError(bool error) {_data_error = error;}
    void setDataError() {setDataError(true);}
    bool isDataError() {return _data_error;}
    
    void setTimeError(bool error) {_time_error = error;}
    void setTimeError() {setTimeError(true);}
    bool isTimeError() {return _time_error;}

    void setIfaceError(bool error) {_iface_error = error;}
    void setIfaceError() {setIfaceError(true);}
    bool isIfaceError() {return _iface_error;}
    
    void setGroup(int group) { _group = group; }
    int  getGroup()           { return _group;  }
    
    void findSimilarity();
    
    void addImplReactionUnit(ReactionUnit &_reaction_impl)
    {
        _impl_reaction_unit = _reaction_impl;
    }

    void addSpecReactionUnit(ReactionUnit &_reaction_spec)
    {
        _spec_reaction_unit = _reaction_spec;
    }

    void addCollapsedReactionUnit(ReactionUnit &_reaction_collapsed)
    {
        _collapsed_reaction_unit = _reaction_collapsed;
    }

    bool getImplReactionUnit(ReactionUnit &impl_reaction_unit)
    {
        if(isCollapsed()) 
        {
            impl_reaction_unit = _impl_reaction_unit;
            return true;
        }
        else return false;
    }

    bool getSpecReactionUnit(ReactionUnit &spec_reaction_unit)
    {
        if(isCollapsed()) 
        {
            spec_reaction_unit = _spec_reaction_unit;
            return true;
        }
        else return false;
    }

    ReactionUnit& getCollapsedReactionUnit()
    {
        return _collapsed_reaction_unit;
    }

    void addCharactristic(bool error_data, bool time_error)
    {
        setDataError(error_data);
        setTimeError(time_error);
    }

protected:
    ReactionUnit _impl_reaction_unit;
    ReactionUnit _spec_reaction_unit;
    ReactionUnit _collapsed_reaction_unit;
    
    bool _collapsed;
    bool _data_error;
    bool _time_error;
    bool _iface_error;
    int  _error;
    int  _group;
};

}}}
