/* Copyright 2010-2012 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: 
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <vector>

#include <hw/message.hpp>
#include <hw/diagnostics/reaction_unit.hpp>

namespace cpptesk {
namespace hw {
namespace diagnostics {

class ReactionSet
{
protected:
    std::vector<ReactionUnit> _reaction_vector;

public:
    ReactionSet() {}
    ~ReactionSet() {}

    ReactionUnit& getReactionUnit(int index)
    {
        return _reaction_vector[index];
    }

    bool isEmpty() { return _reaction_vector.size() == 0; }
    int size() { return _reaction_vector.size(); }

    void addReactionUnit(ReactionUnit _reaction_unit)
    {
        _reaction_unit.setIdentifier(_reaction_vector.size());
        _reaction_vector.push_back(_reaction_unit);
    }

    void addReaction(const ::cpptesk::hw::Reaction &_reaction)
    {
        ReactionUnit reactionUnit(_reaction);
        addReactionUnit(reactionUnit);
    }

    void addReaction(::cpptesk::hw::Reaction &_reaction, int parent_spec, int parent_impl)
    {
        ReactionUnit reactionUnit(_reaction, parent_spec, parent_impl);
        addReactionUnit(reactionUnit);
    }

    void addReactionSet(ReactionSet &_reaction_set, int _parent_1, int _parent_2)
    {
        for(unsigned int i = 0; i < _reaction_set._reaction_vector.size(); i++)
        {
            _reaction_set.getReactionUnit(i).setParents(_parent_1, _parent_2);
            addReactionUnit(_reaction_set.getReactionUnit(i));
        }
    }

    void addReactionSet(ReactionSet &_reaction_set)
    {
        addReactionSet(_reaction_set, -1, -1);
    }

    void initialization(const event_list_t &reactions)
    {
        for(event_list_t::const_iterator i = reactions.begin(); i != reactions.end(); i++)
        {
           const Reaction *reaction = dynamic_cast<const Reaction*>(i->get());
           addReaction(*reaction);
        }
    }

    void clear() { _reaction_vector.clear(); }
};

}}}
