/*
 * Copyright 2012 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK Hardware Edition
// AUTHOR:   Mikhail Chupilko (chupilko@ispras.ru)
// SYNOPSIS: Functions-wrappers for interconnection with VeriTool
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/defs.hpp>

extern "C" {
    extern void vsim_putUInt64Value(const char *reg, uint64_t val);
    extern uint64_t vsim_getUInt64Value(const char *reg);
    extern void vsim_putUInt32Value(const char *reg, uint32_t val);
    extern uint32_t vsim_getUInt32Value(const char *reg);
    extern void vsim_putUInt16Value(const char *reg, uint16_t val);
    extern uint16_t vsim_getUInt16Value(const char *reg);
    extern void vsim_putUInt8Value(const char *reg, uint8_t val);
    extern uint8_t vsim_getUInt8Value(const char *reg);
}

namespace cpptesk {
namespace hw {

template<typename T> void set_value(const char *full_name, T t)
{
    switch(sizeof(T))
    {
        case 1: ::vsim_putUInt8Value(full_name, t); break;
        case 2: ::vsim_putUInt16Value(full_name, t); break;
        case 3: ::vsim_putUInt32Value(full_name, t); break;
        case 4: ::vsim_putUInt32Value(full_name, t); break;
        case 5: ::vsim_putUInt64Value(full_name, t); break;
        case 6: ::vsim_putUInt64Value(full_name, t); break;
        case 7: ::vsim_putUInt64Value(full_name, t); break;
        case 8: ::vsim_putUInt64Value(full_name, t); break;
        default: CPPTESK_ASSERTION(false, "set_value: size of wire is too long");
    }
}

template<typename T> T get_value(const char *full_name)
{
    switch(sizeof(T))
    {
        case 1: return ::vsim_getUInt8Value(full_name); break;
        case 2: return ::vsim_getUInt16Value(full_name); break;
        case 3: return ::vsim_getUInt32Value(full_name); break;
        case 4: return ::vsim_getUInt32Value(full_name); break;
        case 5: return ::vsim_getUInt64Value(full_name); break;
        case 6: return ::vsim_getUInt64Value(full_name); break;
        case 7: return ::vsim_getUInt64Value(full_name); break;
        case 8: return ::vsim_getUInt64Value(full_name); break;
        default: CPPTESK_ASSERTION(false, "get_value: size of wire is too long"); return -1;
    }
}

template<typename T> void set_value(const std::string &full_name, T t)
{
    set_value<T>(full_name.c_str(), t);
}

template<typename T> T get_value(const std::string &full_name)
{
    return get_value<T>(full_name.c_str());
}

}} // namespace cpptesk::hw

