/** @file ccp_main.cpp
 *
 * Available scenarios for verifying the cache coherence protocol.
 * They differ by state abstraction techniques being used.
 *
 * @author Alexander Kamkin (kamkin@ispras.ru)
 */

#include <utils/testreg.h>

#include <ccp_scenario.h>

using namespace cpptesk::examples::ccp;

VectorStateScenario   vector_test;   // FSM state is a tuple of the cache states
MultiSetStateScenario multiset_test; // FSM state is a multi-set of the cache states
SetStateScenario      set_test;      // FSM state is a set of the cache states

#define TEST_OPTS "-nt -tf2 trace.utt2 -uerr=1 --nondeterministic --print-progress"

TEST_REGISTRY_BEGIN
    REGISTER_TEST(vector_test,   engine::fsm, TEST_OPTS)
    REGISTER_TEST(multiset_test, engine::fsm, TEST_OPTS)
    REGISTER_TEST(set_test,      engine::fsm, TEST_OPTS)
TEST_REGISTRY_END

int main(int argc, char **argv)
{
    if(argc > 1)
    {
        return startScenario_TestRegistry(argv[1], LOCAL);
    }

    std::cout << "Usage: " << argv[0] << " <test>"                                 << std::endl;
    std::cout << "There are several tests available:"                              << std::endl;
    std::cout << " * vector_test   - FSM state is a tuple of the cache states;"    << std::endl;
    std::cout << " * multiset_test - FSM state is a multiset of the cache states;" << std::endl;
    std::cout << " * set_test      - FSM state is a set of the cache states."      << std::endl;

    return 0;
}

