/** @file counter_model.h
 * This file contains counter model class description.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#pragma once

#include <hw/model.hpp>

#include <counter_coverage.h>
#include <counter_msg.h>

// header for DPI connection
#include "svdpi.h"

namespace cpptesk {
namespace examples {
namespace counter {

// To have the SystemVerilog Reference Model of Mau Hub connected
typedef struct {
    uint32_t     valid;
} InputSignals;

typedef struct {
    uint32_t     valid;
    uint32_t     data;
} OutputSignals;

typedef struct {
    uint32_t in_id;
    uint32_t in_timemark;
    uint32_t out_id;
    uint32_t out_timemark;
} Requests;

extern InputSignals refinputs;
extern OutputSignals refoutputs;
extern Requests refreqs;
extern char refcurrentstate[255];

extern "C" {
    void init_state(char *state);
    void take_inputs(InputSignals *inputs, Requests *requests);
    void send_outputs(OutputSignals *outputs, Requests *requests);
} // extern "C"

// The model
MODEL(Counter) {
public:
    Counter();
    virtual ~Counter();

    DECLARE_INPUT(iface1);
    DECLARE_OUTPUT(iface2);

    virtual bool isIface1Ready() const;

    enum {NOP, OP1} labels;
    DECLARE_PROCESS(answer_from_ref_model);
    DECLARE_PROCESS(inc_send_to_ref_model);
    DECLARE_STIMULUS(inc);
    DECLARE_REACTION(read_result);

    CounterCoverage coverage;
    void refresh_coverage(void);
};

}}}
