/*Copyright 2010-2012 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Fuzzy Impl Intersection Elimination 
//           (Rule # 11 in C++TESK.ErrorDiagnostics.ru.doc):
//           {[Y1:x]+[Z1:Y2] => [Z1:x]+[Y1:Y2]}
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/diagnostics/rule/rule.hpp>

namespace cpptesk {
namespace hw {
namespace diagnostics {

class FuzzyImplIntersectionEliminationRule: 
      public Rule
{
    public:

    FuzzyImplIntersectionEliminationRule() {}
    virtual ~FuzzyImplIntersectionEliminationRule() {};

    ReactionSet transform(::cpptesk::hw::diagnostics::ReactionUnit &reaction_unit_1, 
                          ::cpptesk::hw::diagnostics::ReactionUnit &reaction_unit_2);
    // Collapse the reactions

    int match(::cpptesk::hw::Reaction reaction_1, 
              ::cpptesk::hw::Reaction reaction_2);
    // Compare the reactions
    

    void weight()
    // Get the weight of operation
    {
    }
    
    void setCollapseRule(::cpptesk::hw::diagnostics::ReactionUnit &reaction_unit_1, 
                         ::cpptesk::hw::diagnostics::ReactionUnit &reaction_unit_2, 
                         int rule, int type)
    {
        reaction_unit_1.setCollapseRule(rule);
        reaction_unit_1.setCollapseType(type);

        reaction_unit_2.setCollapseRule(rule);
        reaction_unit_2.setCollapseType(type);
    }

    void distance()
    {
    }
    
    std::string getRuleDescription()
    {
        return "for UNEXPECTED and INCORRECT reactions: {[Y1:x]+[Z1:Y2] => [Z1:x]+[Y1:Y2]}.";
    }
};

}}}
